@@ -14,7 +14,7 @@
 # $1 full target dir
 # $2 full source luks2 image
 
-function generate()
+generate()
 {
 	# make area 7 access the luks2 header space
 	OFFS=$((2*LUKS2_HDR_SIZE*512-1))
@@ -28,7 +28,7 @@ function generate()
 	lib_mangle_json_hdr0_kill_hdr1
 }
 
-function check()
+check()
 {
 	lib_hdr1_killed || exit 2
 
